@@ -44,11 +44,12 @@ const char *const units[] = {
 /* Get and print status from apcupsd NIS server */
 static int do_pthreads_status(const char *host, int port, const char *par, int flags)
 {
-   int sockfd, n;
+   sock_t sockfd;
+   int n;
    char recvline[MAXSTRING + 1];
    char *line;
 
-   if ((sockfd = net_open(host, NULL, port)) < 0) {
+   if ((sockfd = net_open(host, NULL, port)) == INVALID_SOCKET) {
       fprintf(stderr, "Error contacting apcupsd @ %s:%d: %s\n",
          host, port, strerror(-sockfd));
       return 1;
@@ -141,12 +142,8 @@ int main(int argc, char **argv)
    int flags = 0;
    FILE *cfg;
 
-#ifdef HAVE_MINGW
-   WSA_Init();                   /* init MS networking */
-#endif
-
    // Process standard options
-   char ch;
+   int ch;
    while ((ch = getopt(argc, argv, "f:h:p:u")) != -1)
    {
       switch (ch)
@@ -205,7 +202,7 @@ int main(int argc, char **argv)
       host = argv[optind + 1];
 
    // If still no host, use default
-   if (!host)
+   if (!host || !*host)
       host = DEFAULT_HOST;
 
    // Separate host and port
