@@ -35,6 +35,16 @@ class StatMgr;
 @protocol NSTableViewDataSource <NSObject> @end
 #endif 
 
+// NSUserNotificationCenterDelegate was added in 10.8
+#if __MAC_OS_X_VERSION_MAX_ALLOWED < 1080
+@protocol NSUserNotificationCenterDelegate <NSObject> @end
+#endif
+
+// NSMenuDelegate was added in 10.6
+#if __MAC_OS_X_VERSION_MAX_ALLOWED < 1060
+@protocol NSMenuDelegate <NSObject> @end
+#endif
+
 @interface StatusTableDataSource: NSObject <NSTableViewDataSource>
 {
    NSLock *_mutex;
@@ -64,8 +74,7 @@ class StatMgr;
    row:(int)rowIndex;
 @end
 
-
-@interface AppController: NSObject
+@interface AppController: NSObject <NSMenuDelegate, NSUserNotificationCenterDelegate>
 {
    // Status item shown in the status bar
    NSStatusItem *statusItem;
@@ -133,6 +142,9 @@ class StatMgr;
    // C++ object which handles polling the UPS
    StatMgr *statmgr;
 
+   // If system support Notification Center (10.8 and above)
+   BOOL haveNotifCtr;
+
    // Runloop variables
    BOOL running;
    CFRunLoopRef runLoop;
