@@ -244,37 +244,16 @@ void apctest_error_cleanup(UPSINFO *ups)
  * and exits. It is normally called from the Error_abort
  * define, which inserts the file and line number.
  */
-void apctest_error_out(const char *file, int line, const char *fmt, ...)
+void apctest_error_out(const char *file, int line, const char *fmt, va_list arg_ptr)
 {
    char buf[256];
-   va_list arg_ptr;
    int i;
 
    asnprintf(buf, sizeof(buf),
       "apctest FATAL ERROR in %s at line %d\n", file, line);
    i = strlen(buf);
 
-   va_start(arg_ptr, fmt);
    avsnprintf((char *)&buf[i], sizeof(buf) - i, (char *)fmt, arg_ptr);
-   va_end(arg_ptr);
-
-   pmsg(buf);
-
-   apctest_error_cleanup(core_ups);     /* finish the work */
-}
-
-/*
- * Subroutine error_exit simply prints the supplied error
- * message, cleans up, and exits.
- */
-void apctest_error_exit(const char *fmt, ...)
-{
-   char buf[256];
-   va_list arg_ptr;
-
-   va_start(arg_ptr, fmt);
-   avsnprintf(buf, sizeof(buf), (char *)fmt, arg_ptr);
-   va_end(arg_ptr);
 
    pmsg(buf);
 
@@ -290,7 +269,6 @@ int main(int argc, char *argv[])
 {
    /* Set specific error_* handlers. */
    error_out = apctest_error_out;
-   error_exit = apctest_error_exit;
 
    /*
     * Default config file. If we set a config file in startup switches, it
@@ -300,7 +278,7 @@ int main(int argc, char *argv[])
 
    ups = new_ups();                /* get new ups */
    if (!ups)
-      Error_abort1("%s: init_ipc failed.\n", argv[0]);
+      Error_abort("%s: init_ipc failed.\n", argv[0]);
 
    init_ups_struct(ups);
    core_ups = ups;                 /* this is our core ups structure */
@@ -318,7 +296,7 @@ int main(int argc, char *argv[])
 
    attach_driver(ups);
    if (ups->driver == NULL)
-      Error_abort0("apctest cannot continue without a valid driver.\n");
+      Error_abort("apctest cannot continue without a valid driver.\n");
 
 //   pmsg("Attached to driver: %s\n", ups->driver->driver_name);
 
@@ -330,13 +308,18 @@ int main(int argc, char *argv[])
    pmsg("mode.type = %s\n", ups->mode.long_name);
 
    if (create_lockfile(ups) == LCKERROR) {
-      Error_abort0("Unable to create UPS lock file.\n"
+      Error_abort("Unable to create UPS lock file.\n"
                    "  If apcupsd or apctest is already running,\n"
                    "  please stop it and run this program again.\n");
    }
 
    pmsg("Setting up the port ...\n");
-   setup_device(ups);
+   if (!setup_device(ups))
+   {
+      Error_abort("Unable to open UPS device.\n"
+                  "  If apcupsd or apctest is already running,\n"
+                  "  please stop it and run this program again.\n");
+   }
 
    if (hibernate_ups) {
       pmsg("apctest: bad option, I cannot do a killpower\n");
@@ -1028,16 +1011,41 @@ static void monitor_calibration_progress(int monitor)
    char *ans;
    int count = 6;
    int max_count = 6;
-   char period = '.';
 
    pmsg("Monitoring the calibration progress ...\n"
         "To stop the calibration, enter a return.\n");
 
    for (;;) {
+      int percent;
+      char cmd;
+      bool aborted = false;
+      int retval = 0;
+
+#ifdef HAVE_MINGW
+      // select only works on sockets on win32, so we must poll
+      for (unsigned int i = 0; !aborted && retval == 0 && i < 10; ++i)
+      {
+         while (kbhit())
+         {
+            aborted = true;
+            (void)getch();
+         }
+         if (!aborted)
+         {
+            COMMTIMEOUTS ct;
+            HANDLE h = (HANDLE)_get_osfhandle(ups->fd);
+            ct.ReadIntervalTimeout = MAXDWORD;
+            ct.ReadTotalTimeoutMultiplier = MAXDWORD;
+            ct.ReadTotalTimeoutConstant = 1000;
+            ct.WriteTotalTimeoutMultiplier = 0;
+            ct.WriteTotalTimeoutConstant = 0;
+            SetCommTimeouts(h, &ct);
+            retval = read(ups->fd, &cmd, 1);
+         }
+      }
+#else
       fd_set rfds;
       struct timeval tv;
-      int retval, percent;
-      char cmd;
 
       FD_ZERO(&rfds);
       FD_SET(ups->fd, &rfds);
@@ -1047,9 +1055,64 @@ static void monitor_calibration_progress(int monitor)
       errno = 0;
 
       retval = select((ups->fd) + 1, &rfds, NULL, NULL, &tv);
+      if (retval == -1 && (errno == EINTR || errno == EAGAIN))
+      {
+         continue;
+      }
+      else if (retval != 0)
+      {
+         if (FD_ISSET(STDIN_FILENO, &rfds))
+         {
+            /* *ANY* user input aborts the calibration */
+            read(STDIN_FILENO, &cmd, 1);
+            aborted = true;
+         }
+         else if(FD_ISSET(ups->fd, &rfds))
+         {
+            // UPS char
+            retval = read(ups->fd, &cmd, 1);
+         }
+      }
+#endif
 
-      switch (retval) {
-      case 0:
+      if (retval == -1)
+      {
+         pmsg("\nselect/read failure.\n");
+         terminate_calibration(0);
+         return;
+      }
+
+      if (aborted)
+      {
+         pmsg("\nUser input. Terminating calibration ...\n");
+         terminate_calibration(0);
+         return;
+      }
+
+      if (retval != 0)
+      {
+         if (cmd == '$') {
+            pmsg("\nBattery Runtime Calibration terminated by UPS.\n");
+            pmsg("Checking estimated runtime ...\n");
+            ans = smart_poll('j', ups);
+            if (*ans >= '0' && *ans <= '9') {
+               int rt = atoi(ans);
+
+               pmsg("Remaining runtime is %d minutes\n", rt);
+            } else {
+               pmsg("Unexpected response from UPS: %s\n", ans);
+            }
+            return;
+            /* ignore normal characters */
+         } else if (cmd == '!' || cmd == '+' || cmd == ' ' ||
+            cmd == '\n' || cmd == '\r') {
+            continue;
+         } else {
+            pmsg("\nUPS sent: %c\n", cmd);
+         }
+      }
+      else
+      {
          if (++count >= max_count) {
             ans = smart_poll('f', ups); /* Get battery charge */
             percent = (int)strtod(ans, NULL);
@@ -1078,49 +1141,8 @@ static void monitor_calibration_progress(int monitor)
             }
             count = 0;
          } else {
-            write(STDOUT_FILENO, &period, 1);
-         }
-         continue;
-
-      case -1:
-         if (errno == EINTR || errno == EAGAIN)
-            continue;
-
-         pmsg("\nSelect error. ERR=%s\n", strerror(errno));
-         return;
-
-      default:
-         break;
-      }
-
-      /* *ANY* user input aborts the calibration */
-      if (FD_ISSET(STDIN_FILENO, &rfds)) {
-         read(STDIN_FILENO, &cmd, 1);
-         pmsg("\nUser input. Terminating calibration ...\n");
-         terminate_calibration(0);
-         return;
-      }
-
-      if (FD_ISSET(ups->fd, &rfds)) {
-         read(ups->fd, &cmd, 1);
-         if (cmd == '$') {
-            pmsg("\nBattery Runtime Calibration terminated by UPS.\n");
-            pmsg("Checking estimated runtime ...\n");
-            ans = smart_poll('j', ups);
-            if (*ans >= '0' && *ans <= '9') {
-               int rt = atoi(ans);
-
-               pmsg("Remaining runtime is %d minutes\n", rt);
-            } else {
-               pmsg("Unexpected response from UPS: %s\n", ans);
-            }
-            return;
-            /* ignore normal characters */
-         } else if (cmd == '!' || cmd == '+' || cmd == ' ' ||
-            cmd == '\n' || cmd == '\r') {
-            continue;
-         } else {
-            pmsg("\nUPS sent: %c\n", cmd);
+            printf(".");
+            fflush(stdout);
          }
       }
    }
@@ -2388,7 +2410,7 @@ static void print_eeprom_values(UPSINFO *ups)
    pmsg("Doing prep_device() ...\n");
    prep_device(ups);
    if (!ups->UPS_Cap[CI_EPROM])
-      Error_abort0("Your model does not support EPROM programming.\n");
+      Error_abort("Your model does not support EPROM programming.\n");
 
    if (ups->UPS_Cap[CI_REVNO])
       locale1 = *(ups->firmrev + strlen(ups->firmrev) - 1);
@@ -2401,7 +2423,7 @@ static void print_eeprom_values(UPSINFO *ups)
       locale2 = 0;
 
    if (locale1 == locale2 && locale1 == 0)
-      Error_abort0("Your model does not support EPROM programming.\n");
+      Error_abort("Your model does not support EPROM programming.\n");
 
    if (locale1 == locale2)
       locale = locale1;
