@@ -217,7 +217,7 @@ int ApcSmartUpsDriver::getline(char *s, int len)
          } else if (errno == EBADF) {
             return FAILURE;               /* We're probably shutting down */
          }
-         Error_abort1("Select error on UPS FD. %s\n", strerror(errno));
+         Error_abort("Select error on UPS FD. %s\n", strerror(errno));
          break;
       default:
          break;
@@ -440,7 +440,7 @@ bool ApcSmartUpsDriver::read_volatile_data()
     again:
       answer = smart_poll(_ups->UPS_Cmd[CI_STATUS]);
       Dmsg(80, "Got CI_STATUS: %s\n", answer);
-      strncpy(status, answer, sizeof(status));
+      strlcpy(status, answer, sizeof(status));
 
       /*
        * The Status command may return "SM" probably because firmware
