@@ -33,8 +33,8 @@
 using namespace Snmp;
 
 SnmpEngine::SnmpEngine() :
-   _socket(-1),
-   _trapsock(-1),
+   _socket(INVALID_SOCKET),
+   _trapsock(INVALID_SOCKET),
    _reqid(0)
 {
 }
@@ -82,7 +82,7 @@ bool SnmpEngine::Open(const char *host, unsigned short port, const char *comm)
 
    // Get a UDP socket
    _socket = socket(PF_INET, SOCK_DGRAM, 0);
-   if (_socket == -1)
+   if (_socket == INVALID_SOCKET)
    {
       perror("socket");
       return false;
@@ -99,7 +99,7 @@ bool SnmpEngine::Open(const char *host, unsigned short port, const char *comm)
    {
       perror("bind");
       close(_socket);
-      _socket = -1;
+      _socket = INVALID_SOCKET;
       return false;
    }
 
@@ -109,7 +109,7 @@ bool SnmpEngine::Open(const char *host, unsigned short port, const char *comm)
 bool SnmpEngine::EnableTraps()
 {
    _trapsock = socket(PF_INET, SOCK_DGRAM, 0);
-   if (_trapsock == -1)
+   if (_trapsock == INVALID_SOCKET)
    {
       perror("socket");
       return false;
@@ -125,7 +125,7 @@ bool SnmpEngine::EnableTraps()
    {
       perror("bind");
       close(_trapsock);
-      _trapsock = -1;
+      _trapsock = INVALID_SOCKET;
       return false;
    }
 
@@ -134,16 +134,16 @@ bool SnmpEngine::EnableTraps()
 
 void SnmpEngine::Close()
 {
-   if (_socket != -1)
+   if (_socket != INVALID_SOCKET)
    {
       close(_socket);
-      _socket = -1;
+      _socket = INVALID_SOCKET;
    }
 
-   if (_trapsock != -1)
+   if (_trapsock != INVALID_SOCKET)
    {
       close(_trapsock);
-      _trapsock = -1;
+      _trapsock = INVALID_SOCKET;
    }
 }
 
@@ -313,7 +313,7 @@ VbListMessage *SnmpEngine::perform(VbListMessage *req)
 
 TrapMessage *SnmpEngine::TrapWait(unsigned int msec)
 {
-   if (_trapsock == -1)
+   if (_trapsock == INVALID_SOCKET)
       return NULL;
 
    return (TrapMessage*)rspwait(msec, true);
@@ -346,7 +346,7 @@ Message *SnmpEngine::rspwait(unsigned int msec, bool trap)
    static unsigned char data[8192];
    struct sockaddr_in fromaddr;
 
-   int sock = trap ? _trapsock : _socket;
+   sock_t sock = trap ? _trapsock : _socket;
 
    // Calculate exit time
    struct timeval exittime;
