@@ -20,8 +20,8 @@
 #define BALLOONMGR_H
 
 #include <windows.h>
-#include <string>
-#include <vector>
+#include "astring.h"
+#include "alist.h"
 
 class BalloonMgr
 {
@@ -43,16 +43,16 @@ private:
 
    struct Balloon {
       HWND hwnd;
-      std::string title;
-      std::string text;
+      astring title;
+      astring text;
    };
 
-   std::vector<Balloon> _pending;
+   alist<Balloon>       _pending;
    HANDLE               _mutex;
+   bool                 _exit;
    bool                 _active;
    HANDLE               _event;
    HANDLE               _timer;
-   bool                 _exit;
    struct timeval       _time;
    HANDLE               _thread;
 };
